'use strict'

const execSync = require('child_process').execSync
const setup = require('./setup')
const managers = {
  npm: {
    name: 'npm',
    install: 'install'
  },
  cnpm: {
    name: 'cnpm',
    install: 'install'
  },
  yarn: {
    name: 'yarn',
    install: 'add'
  }
}

/**
 * 包管理工具
 */
exports.managers = managers

/**
 * 检查模块是否已经安装
 */
exports.isInstalled = function (moduleName, checkPath) {
  const cwd = process.cwd()

  process.chdir(checkPath)
  try {
    execSync('node -e "require.resolve(\'' + moduleName + '\')"', {
      stdio: 'ignore',
      env: process.env
    })
    return true
  } catch (err) {
    return false
  } finally {
    process.chdir(cwd)
  }
}

/**
 * 安装模块
 */
exports.install = function (moduleName, installPath) {
  const cwd = process.cwd()
  const profile = setup.getProfile()
  const pm = profile.pm
  const manager = managers[pm]

  process.chdir(installPath)
  execSync([pm, manager.install, moduleName].join(' '), {
    stdio: 'inherit'
  })
  process.chdir(cwd)
}

/**
 * 使用包管理器
 */
exports.use = function (pm) {
  const profile = setup.getProfile()
  profile.pm = pm
  setup.updateProfile(profile)
}
