'use strict'

const path = require('path')
const validate = require('validate-npm-package-name')
const logger = require('../util/logger')
const pm = require('./package-manager')
const setup = require('./setup')
const profile = setup.getProfile()

class PluginManager {
  constructor (options) {
    this.context = options.context
    this.installedPlugins = []
    setup.addModulesPath(options.modulesPath)
  }

  install (src, options) {
    let plugin = null
    const context = this.context

    try {
      plugin = typeof src === 'string'
        ? require(src)
        : src

      if (plugin) {
        if (typeof plugin === 'function') {
          plugin = {
            install: plugin
          }
        }
        plugin.install && plugin.install(context, options)
        this.installedPlugins.push(plugin)
      }
    } catch (err) {
      logger.error('%s load faild', src)
      throw err
    }
  }

  resolve (options) {
    const pluginOptions = options.options
    const cwd = process.cwd()
    let name = options.name

    if (validate(name).errors) {
      if (name.indexOf('@/') === 0) {
        name = path.resolve(profile.basedir, name.replace('@/', ''))
      } else {
        name = path.resolve(cwd, name)
      }
    } else {
      name = 'workflow-plugin-' + name

      if (!pm.isInstalled(name, cwd)) {
        logger.warn('installing %s', name)
        pm.install(name, profile.basedir)
      }
    }

    this.install(require.resolve(name), pluginOptions)
  }

  parse (values) {
    values.forEach(plugin => {
      let name = plugin
      let options = {}

      if (Array.isArray(plugin)) {
        name = plugin[0]
        options = plugin[1] || options
      }

      if (typeof name === 'string') {
        this.resolve({
          name: name,
          options: options
        })
      } else {
        this.install(name, options)
      }
    })
  }
}

module.exports = PluginManager
