'use strict'

const fs = require('fs')
const path = require('path')
const pm = require('./package-manager')
const config = require('./config')

const PROFILE_PATH = config.PROFILE_PATH
const BASE_DIR = config.BASE_DIR

/**
 * 写入profile
 * @param {Object} data
 */
function updateProfile (data) {
  fs.writeFileSync(PROFILE_PATH, JSON.stringify(data, null, 2))
}
exports.updateProfile = updateProfile

/**
 * 添加模块查找目录
 * @param {Array|String} paths
 */
function addModulesPath (paths) {
  paths = [].concat(paths, process.env.NODE_PATH || [])

  process.env.NODE_PATH = paths.join(path.delimiter)
  require('module').Module._initPaths()
}
exports.addModulesPath = addModulesPath

/**
 * 获取配置对象
 */
function getProfile () {
  return require(PROFILE_PATH)
}
exports.getProfile = getProfile

/**
 * 确保存在profile和basedir
 */
exports.init = function () {
  let profile = null

  // 创建profile文件
  if (fs.existsSync(PROFILE_PATH)) {
    profile = getProfile()
  } else {
    updateProfile(profile = {
      basedir: BASE_DIR,
      pm: pm.managers.npm.name
    })
  }

  // 创建基础目录
  if (!fs.existsSync(profile.basedir)) {
    fs.mkdirSync(profile.basedir)
  }

  // 创建package.json
  const pkgFile = path.join(profile.basedir, 'package.json')
  if (!fs.existsSync(pkgFile)) {
    fs.writeFileSync(pkgFile, JSON.stringify({
      //
    }))
  }
}
