'use strict'

const EventEmitter = require('events')
const gulp = require('gulp')
const webpack = require('webpack')
const merge = require('webpack-merge')
const express = require('express')
const hashFilename = require('../webpack/hash-filename')
const generateCssLoaders = require('../webpack/parse-config').generateCssLoaders

const defaultConfig = {
  output: {
    path: './dist',
    filename: 'js/[name].js'
  },
  server: true,
  hash: true,
  extractCSS: true,
  minimize: true,
  commonChunk: false,
  template: false,
  minifyImages: {
    jpgProgressive: true,
    quality: 70,
    pngQuality: [0.65, 0.90],
    pngSpeed: 4,
    optipngEnabled: false,
    gifInterlaced: false,
    exclude: [/raw\//],
    loaderRawOptions: null
  },
  transpileDependencies: []
}

class Workflow extends EventEmitter {
  constructor (options) {
    super()
    this.gulp = gulp
    this.webpack = webpack
    this.express = express
    this.server = null
    this.compiler = null
    this.compilerOptions = {}
    this.cwd = options.cwd
    this.env = options.env
    this.task = options.task
    this.config = merge.strategy({
      'minifyImages.exclude': 'replace',
      'minifyImages.pngQuality': 'replace'
    })(defaultConfig, options.config)
    if (this.config.minifyImages === true) {
      this.config.minifyImages = defaultConfig.minifyImages
    }
  }

  initServer () {
    return (this.server = express())
  }

  initCompiler (compilerOptions) {
    this.compiler = webpack(compilerOptions)
    this.emit('compilerReady', this.compiler)
    return this.compiler
  }

  setCompilerOptions (options) {
    this.compilerOptions = merge.strategy({
      entry: 'replace'
    })(this.compilerOptions, options)
  }

  generateCssLoaders (options) {
    return generateCssLoaders(this.config, options)
  }

  hashFilenameIfNeed (filename, placeholder) {
    if (this.config.hash) {
      return hashFilename(filename, placeholder)
    }
    return filename
  }

  setEnv (env) {
    this.env = process.env.NODE_ENV = env
  }
}

module.exports = Workflow
