'use strict'

const del = require('del')
const logger = require('../util/logger')
const compilerHandler = require('../webpack/handler')

const taskName = 'build'
const env = require('../lib/config').ENV_PRODUCTION

module.exports = function (workflow, options) {
  var gulp = workflow.gulp

  if (workflow.task === taskName) {
    workflow.setEnv(env)
    init(workflow)
  }

  gulp.task(taskName, gulp.series(rmdir, build))

  function rmdir (done) {
    const output = workflow.compilerOptions.output
    if (output && output.path) {
      logger.info('cleaning %s', output.path)
      return del([output.path], { force: true })
    }
    done()
  }

  function build (done) {
    const webpackConfig = workflow.compilerOptions
    let compiler = null

    webpackConfig.output.filename = workflow.hashFilenameIfNeed(webpackConfig.output.filename, 'chunkhash')
    compiler = workflow.initCompiler(webpackConfig)

    logger.info('building for production...')
    compiler.run((err, stats) => {
      if (compilerHandler(err, stats)) {
        done()
      }
    })
  }
}

function init (workflow) {
  const config = workflow.config
  const webpackConfig = require('../webpack/config.build')(config, env)
  workflow.setCompilerOptions(webpackConfig)
}
